import urllib.request
import logging
from datetime import datetime, timedelta
from defence360agent.api.server import API
from defence360agent.internals.iaid import (
    IndependentAgentIDAPI,
    IAIDTokenError,
)
from defence360agent.rpc_tools.utils import run_in_executor_decorator

logger = logging.getLogger(__name__)

CACHE_TTL = timedelta(minutes=10)


class AnalystCleanupAPI(API):
    CLEANUP_ALLOWED_URL_TEMPLATE = (
        "{base}/api/analyst-assisted-cleanup/is-allowed"
    )

    # Cache for the cleanup allowed check
    _cache = {
        "result": None,
        "timestamp": datetime.min,  # Initialize with minimum datetime
    }

    @classmethod
    async def check_cleanup_allowed(cls):
        """Check if analyst cleanup is allowed for this installation"""
        current_time = datetime.now()
        if (
            cls._cache["result"] is not None
            and current_time - cls._cache["timestamp"] < CACHE_TTL
        ):
            return cls._cache["result"]

        try:
            request = urllib.request.Request(
                cls.CLEANUP_ALLOWED_URL_TEMPLATE.format(base=cls._BASE_URL),
                headers={"X-Auth": await IndependentAgentIDAPI.get_token()},
                method="GET",
            )
        except IAIDTokenError:
            return False
        else:
            result = await cls._check_allowed(request)
            cls._cache["result"] = result
            cls._cache["timestamp"] = datetime.now()
            return result

    @classmethod
    @run_in_executor_decorator
    def _check_allowed(cls, request):
        """Execute the actual request in executor"""
        try:
            result = cls.request(request)
            return result.get("result", False)
        except Exception as e:
            logger.error("Failed to check cleanup permission: %s", e)
            # NOTE:
            # If the API returns an error, the request should be allowed
            # (to prevent extra sales trips due to API instability).
            # Ref: https://cloudlinux.slite.com/app/docs/Fhb2ASESxb9111
            return True
