import asyncio
import re
import tempfile
import subprocess
import urllib.request
import os

from logging import getLogger
from urllib.error import URLError
from pathlib import Path


logger = getLogger(__name__)

ANALYST_PUB_KEY_URL = (
    "https://repo.cloudlinux.com/cloudlinux/cldoctor/support-ssh-access-key.sh"
)
SSH_CONFIG_PATH = Path("/etc/ssh/sshd_config")
SSH_CONFIG_DIR = Path("/etc/ssh/sshd_config.d")


async def get_ssh_port():
    """
    Detect SSH port from config and its overrides.
    Searches configs in reverse order to find the last override first.
    """
    port = 22  # default port
    try:
        # Collect and sort config files
        config_files = [SSH_CONFIG_PATH]

        if SSH_CONFIG_DIR.exists():
            config_files.extend(sorted(SSH_CONFIG_DIR.glob("*.conf")))

        # Process files
        for config_file in reversed(config_files):
            try:
                for line in config_file.read_text().splitlines():
                    line = line.strip()
                    if line.startswith("Port ") and not line.startswith("#"):
                        try:
                            # return first match
                            # since we are searching backwards
                            port = int(line.split()[1])
                            return port
                        except (IndexError, ValueError):
                            continue
            except IOError as e:
                logger.warning(f"Failed to read {config_file}: {e}")
                continue
    except Exception as e:
        logger.warning(f"Failed to get SSH port: {e}")
    finally:
        return port


async def check_ssh_connection(port=22):
    """Test if port is actually an SSH port by checking the server banner"""
    try:
        reader, writer = await asyncio.open_connection("127.0.0.1", port)
        try:
            banner = await asyncio.wait_for(reader.readline(), timeout=5.0)
            banner = banner.decode("utf-8", errors="ignore").strip()

            if re.match(r"^SSH-[12]\.", banner):
                logger.info(
                    f"Port {port} is confirmed as SSH (banner: {banner})"
                )
                return True
            else:
                logger.warning(
                    f"Port {port} is open but not SSH (got: {banner})"
                )
                return False

        except asyncio.TimeoutError:
            logger.warning(f"Timeout waiting for SSH banner on port {port}")
            return False
        finally:
            writer.close()
            await writer.wait_closed()

    except (ConnectionRefusedError, OSError) as e:
        logger.warning(f"Failed to connect to port {port}: {e}")
        return False
    except Exception as e:
        logger.warning(f"Unexpected error checking SSH port {port}: {e}")
        return False


async def install_pub_key(username="root"):
    """Install analyst public key for the user
    downloads the script provided in ANALYST_PUB_KEY_URL
    and executes it providing the username in std in of the script
    """
    try:
        # Download the script to a temporary file
        with tempfile.NamedTemporaryFile(
            mode="w+", suffix=".sh", delete=True
        ) as temp_script:
            try:
                script_content = (
                    urllib.request.urlopen(ANALYST_PUB_KEY_URL).read().decode()
                )
            except urllib.error.URLError as e:
                logger.error(f"Failed to download script: {e}")
                return False

            temp_script.write(script_content)
            temp_script.flush()

            # Make the script executable
            os.chmod(temp_script.name, 0o700)

            # If not running as root, fail
            if os.geteuid() != 0:
                logger.error("Script must be run as root")
                return False

            # Prepare input based on username
            input_data = ""
            if username == "root":
                input_data = "1\n"  # Select root option
            else:
                input_data = (  # Select custom user + confirm
                    f"2\n{username}\ny\n"
                )

            if os.path.exists(f"/home/{username}/.ssh/authorized_keys"):
                input_data += "n\n"  # Don't create authorized_keys if exists
            else:
                input_data += "y\n"  # Create authorized_keys if doesn't exist

            # Run the script
            process = subprocess.Popen(
                ["bash", temp_script.name],
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                text=True,
            )

            stdout, stderr = process.communicate(input=input_data)

            if process.returncode != 0:
                logger.error(f"Script execution failed: {stderr}")
                return False

            # Check if key was added successfully
            if "OK: Key added successfully" in stdout:
                logger.info(f"Successfully installed key for user {username}")
                return True
            else:
                logger.error(f"Failed to add key: {stdout}")
                return False

    except Exception as e:
        logger.error(f"Failed to install public key: {e}")
        return False
